%% Design Examples Using Least Square IIR Filter Optimization Program

%% Example 1: Low Order Lowpass Filter with Approximately Linear Passband Phase (M=4, N=4)

M=4;
N=4;
tau=5;
om=pi*[linspace(0,0.2,20),linspace(0.4,1,60)];
D=[exp(-1i*om(1:20)*tau),zeros(1,60)];
W=ones(1,80);
[b,a,e]=mpiir_l2(M,N,om,D,W,0.98);

%%
h=fvtool(b,a);
axis([0 1 -60 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.2(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0 0.2 -0.4 0.4])
%%
% Figure 5.2(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau)));
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0 0.2 -0.04 0.04]);
%%
% Figure 5.2(c)
%%
h=fvtool(b,a);
set(h,'Analysis','grpdelay')
title('Passband Group Delay (samples)')
axis([0 0.2 4.5 6.5])
%%
% Figure 5.2(d)
%%
h=fvtool(b,a);
set(h,'Analysis','polezero')
title('Poles (x) and Zeros (o)')
%%
% Figure 5.3(a)

%% Example 2: Lowpass Filter with Approximately Linear Passband Phase and Magnitude Constraints (M=15, N=15)

M=15;
N=15;
tau=15;
om=pi*[linspace(0,0.4,40),linspace(0.56,1,44)];
D=[exp(-1i*om(1:40)*tau),zeros(1,44)];
W=[ones(1,40),100*ones(1,44)];
[b,a,e]=mpiir_l2(M,N,om,D,W,0.8263);

%%
h=fvtool(b,a);
axis([0 1 -90 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.4(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0 0.4 -0.1 0.1])
%%
% Figure 5.4(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau)));
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0 0.4 -0.015 0.015]);
%%
% Figure 5.4(c)
%%
h=fvtool(b,a);
set(h,'Analysis','grpdelay')
title('Passband Group Delay (samples)')
axis([0 0.4 14.8 15.5])
%%
% Figure 5.4(d)
%%
h=fvtool(b,a);
set(h,'Analysis','polezero')
title('Poles (x) and Zeros (o)')
%%
% Figure 5.5(a)

%% Example 3: Highpass Filter with Approximately Linear Passband Phase (M=14, N=6)

M=14;
N=6;
tau=12;
om=pi*[linspace(0,0.475,50),linspace(0.525,1,50)];
D=[zeros(1,50),exp(-1i*om(51:100)*tau)];
W=ones(1,100);
[b,a,e]=mpiir_l2(M,N,om,D,W,0.9276);

%%
h=fvtool(b,a);
axis([0 1 -50 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.6(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0.53 1 -0.2 0.2])
%%
% Figure 5.6(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,mod(phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau))+pi,2*pi)-pi);
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0.5 1 -0.06 0.06]);
%%
% Figure 5.6(c)
%%
h=fvtool(b,a);
set(h,'Analysis','grpdelay')
title('Passband Group Delay (samples)')
axis([0.53 1 10 14])
%%
% Figure 5.6(d)
%%
h=fvtool(b,a);
set(h,'Analysis','polezero')
title('Poles (x) and Zeros (o)')
axis([-1.5 1.5 -1.5 1.5])
%%
% Figure 5.7(a)

%% Example 4: Narrow Bandpass Filter with Approximately Linear Passband Phase (M=20, N=8)

M=20;
N=8;
tau=20;
om=pi*[linspace(0,0.36,36),linspace(0.4,0.5,10),linspace(0.54,1,46)];
D=[zeros(1,36),exp(-1i*om(37:46)*tau),zeros(1,46)];
W=[100*ones(1,36),ones(1,10),100*ones(1,46)];
[b,a,e]=mpiir_l2(M,N,om,D,W,0.98);

%%
h=fvtool(b,a);
axis([0 1 -80 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.9(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0.53 1 -0.2 0.2])
%%
% Figure 5.9(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,mod(phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau))+pi,2*pi)-pi);
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0.4 0.5 -0.15 0.15]);
%%
% Figure 5.9(c)

%% Example 5: IIR Equalization and Anti-Aliasing Filter (M=20, N=4)
% In this example we want an IIR filter that equalizes circuit
% representing analog filter of order 3.

tau=35;M=20;N=4;r=0.97;
om=pi*[linspace(0,1/16,20),linspace(3/16,1,260)];
Dt=[exp(-1i*om(1:20)*tau),zeros(1,260)];
Wt=[ones(1,20),15*ones(1,260)];
z=[];p=[-0.6493,1/(-0.3246-1i*1.0325),1/(-0.3246+1i*1.0325)];k=0.7606;
[num,den]=zp2tf(z,p,k);
Ha=freqs(num,den,om*16/pi);
D=Dt./Ha;W=Wt.*abs(Ha).^2;
[b,a]=mpiir_l2(M,N,om,D,W,r);

% plot graphs
N=8192;
w=pi*linspace(0,1,N);
Ha=freqs(num,den,w*16/pi);
Hd=freqz(b,a,w);
H=Ha.*Hd;
Hlog=20*log10(abs(H));
plot(w/pi,Hlog);
grid on;
title('Magnitude of Frequency Response (dB)');
xlabel('w/pi');
axis([0 1 -100 5]);
%%
% Figure 5.11(a)
%%
plot(w/pi,Hlog);
grid on;
title('Passband Magnitude (dB)');
xlabel('w/pi');
axis([0 1/16 -0.03 0.03]);
%%
% Figure 5.11(b)
%%
plot(w/pi,phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau)));
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0 1/16 -0.004 0.004]);
%%
% Figure 5.11(c)
%%
plot(w(1:end-1)/pi,-diff(phase(H))*N/pi);
grid on;
title('Passband Group Delay (samples)');
xlabel('w/pi');
axis([0 1/16 34.55 35.55]);
%%
% Figure 5.11(d)

%% Example 6: Comparison to a cascade of Cauer Lowpass Filter and and Phase Equalizer (M=10, N=6)
% In this example we design Lowpass filter with approximately linear phase
% in passband (instead of combination of Cauer Lowpass Filter and and Phase Equalizer)

M=10;N=6;
tau=8.5;
om=pi*[linspace(0,0.5,50),linspace(0.6,1,40)];
D=0.5*(1+10^(-0.5/20))*[exp(-1i*om(1:50)*tau) zeros(1,40)];
W=[ones(1,50) 10*ones(1,40)];
[b,a]=mpiir_l2(M,N,om,D,W,0.92);

%%
h=fvtool(b,a);
axis([0 1 -60 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.12(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0 0.55 -0.55 0.05])
%%
% Figure 5.12(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau)));
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0 0.5 -0.06 0.06]);
%%
% Figure 5.12(c)
%%
h=fvtool(b,a);
set(h,'Analysis','grpdelay')
title('Passband Group Delay (dB)')
axis([0 0.5 7 17])
%%
% Figure 5.12(d)
%%
h=fvtool(b,a);
set(h,'Analysis','polezero')
title('Poles (x) and Zeros (o)')
%%
% Figure 5.13(a)

%% Example 7: Comparison to a Coupled Allpass Filters (M=40, N=6)

tau=37;M=40;N=6;r=0.98;
om=pi*[linspace(0,0.2,20),linspace(0.23,1,77)];
D=[exp(-1i*om(1:20)*tau),zeros(1,77)];
W=[ones(1,20),1000*ones(1,77)];
[b,a]=mpiir_l2(M,N,om,D,W,r);

%%
h=fvtool(b,a);
axis([0 1 -80 5])
title('Magnitude of Frequency Response (dB)')
%%
% Figure 5.14(a)
%%
h=fvtool(b,a);
title('Passband Magnitude Response (dB)')
axis([0 0.22 -1.5 0.5])
%%
% Figure 5.14(b)
%%
N=8192;
w=pi*linspace(0,1,N);
H=freqz(b,a,w);
plot(w/pi,phase(H)-phase(exp(-1i*pi/N*(0:N-1)*tau)));
grid on;
title('Passband Phase Error (radians)');
xlabel('w/pi');
axis([0 0.22 -0.2 0.6]);
%%
% Figure 5.12(c)
%%
h=fvtool(b,a);
set(h,'Analysis','grpdelay')
title('Passband Group Delay (dB)')
axis([0 0.2 20 50])
%%
% Figure 5.14(d)
%%
h=fvtool(b,a);
set(h,'Analysis','polezero')
title('Poles (x) and Zeros (o)')
%%
% Figure 5.15(a)
