function env = envelope(E,om,omedge,mode)
% env = envelope(E,om,omedge,mode)
% generates envelope of real-valued function E on the grid om
% in each band specified by the band edges in omedge;
% mode=l : envelope is piecewise constant
% mode=2 : envelope is piecewise linear
%
% Author: Mathias C. Lang, Vienna University of Technology, Aug. 98

E=E(:); om=om(:); omedge=omedge(:); L = length(E); env = zeros(L,1);
del=[] ;
for i=1:length(omedge),
    if ~any(find(omedge(i)==om)), del=[del;i]; end
end
if ~isempty(del), omedge(del)= []; end
Nbands = length(omedge)/2;
for k=1 : Nbands ,
    I = find(om>=omedge(2*k-1) & om<=omedge(2*k));
    Imax = locmax(E(I)); Imax = I(Imax);
    Emax = E(Imax); ommax = om(Imax);
    Id = locmax(-E(I)); Id = I(Id);
    if mode == 1,
        if Id(1)~=I(1), Id=[I(1);Id]; end
        if Id(end) ~=I(end), Id=[Id;I(end)]; end
        for l=1:length(Id)-1,
            env(Id(l):Id(l+1)) = Emax(l);
        end
    elseif mode == 2,
        if Imax(1)~=I(1),
            Imax= [I(1) ; Imax]; Emax= [Emax(1) ; Emax] ;
            ommax= [om(I(1)) ;ommax];
        end
        if Imax(end)~=I(end),
            Imax=[Imax;I(end)]; Emax=[Emax;Emax(end)];
            ommax=[ommax;om(I(end))] ;
        end
        dE_dom = diff(Emax)./diff(ommax);
        for l=1:length(Emax)-1,
            Il = Imax(l):Imax(l+1);
            env(Il) = dE_dom(l)*(om(Il)-ommax(l)) + Emax(l);
        end
    end
end