function [L,d] = invtoep(t)
% function [L,d] = invtoep(t)
% Cholesky factorization of the inverse of the positive
% definite Hermitian Toeplitz matrix T = toeplitz(t) with
% its first row given by t using Levinson's algorithm:
% inv(T) = L*diag(d)*L'

t = t(:); n = length(t);
x = 1; alpha = t(1);
L = eye(n,n); d = zeros(n,1); d(n)=1/alpha;
for m = 1:n-1,
	k = -(t(m+1:-1:2)'*x)/alpha;
	x = [x;0] + k*flipud([conj(x);0]);
	alpha = alpha*(1 - abs(k)^2);
	L(n-m:n,n-m) = x;
	d(n-m) = 1/alpha;
end