function [x, alpha] = levdurb (t)
% function [x,alpha] = levdurb(t)
% Levinson-Durbin Algorithm
% t: autocorrelation sequence
% x: AR-model/prediction error filter coefficients
% alpha: driving noise/error power

t = t(:); n = length(t);
x = 1; alpha = t(1);
for m = 1:n-1,
    k = -(t(m+1:-1:2)'*x)/alpha;
    x = [x;0] + k*flipud([conj(x);0]);
    alpha = alpha*(1 - abs(k)^2);
end
