function h = lslevin(N,om,D,W)
% h = lslevin(H,om,D,W)
% Complex Least Squares FIR filter design using Levinson's algorithm
%
% h     filter impulse response
% M     filter length
% 01    frequency grid (0 <= om <= pi)
% D     complex desired frequency response on the grid om
% V    positive weighting function on the grid om
%
% example: length 61 bandpass, band edges [.23,.3,.5,.57]*pi,
% weighting 1 in passband and 10 in stopbands, desired passband
% group delay 20 samples
%
% om=pi*[linspace(0, .23,230) ,linspace( .3, .5,200) ,linspace(.57,1,430)] ;
% D= [zeros(1,230) ,exp(-j*om(231:430)*20),zeros(1,430)] ;
% H= [10*ones(1,230) .ones(1,200) ,10*ones( 1,430)] ;
% h = lslevin(61,om,D,H);
%
% Author: Mathias C. Lang, Vienna University of Technology, July 1998
om = om(:); D = D(:); W = W(:); L = length(om);
DR = real(D); DI = imag(D); a = zeros(N,1); b = a;

% Set up vectors for quadratic objective function
% (avoid building matrices)
dvec = D; evec = ones(L,1); el = exp(j*om);
for i=1:N,
    a(i) = W.'*real(evec);
    b(i)= W.'*real(dvec);
    evec = evec.*el; dvec = dvec.*el;
end

a=a/L; b=b/L;

% Compute weighted 12 solution h = levin(a,b);
h=levin(a,b);


% filter shown in Figure 2.3 has been designed using the commands
% om=pi*[linspace(0,.295,1500),linspace(.3,.5,1000),linspace(.505,1,2500)];
% D=[zeros(1,1500),exp(-j*om(1501:2500)*700),zeros(1,2500)];
% W=[100*ones(1,1500),ones(1,1000),100*ones(1,2500)];
% h=lslevin(1401,om,D,W);
