function h = mpls(N,om,m,tau,wm,wp)
% h = mpls(N,om,m,tau,wm,wp)
% Complex Least Squares Design of FIR Filters with independent
% weighting of magnitude and phase errors
%
% h: filter impulse response
% N: filter length
% om: band edges (0 <= om <= pi)
% m: desired magnitude per band (constant)
% tau: desired group delay per band (constant)
% (stopband values irrelevant)
% wm: magnitude weighting function per band (constant)
% wp: phase weighting function per band (constant)
% (stopband values irrelevant)
%
% example:
% om=pi* [0 .4 .5 1]; m= [1 OJ; tau= [20 OJ; wm= [1 100J; wp= [.1 OJ;
% h=mpls(61,om,m,tau,wm,wp);
%
% Author: Mathias C. Lang, Vienna University of Technology, March '98
om=om(:); m=m(:); tau=tau(:); wm=wm(:); wp=wp(:);
% Compute objective function matrices Q(NxN) and q (Nxl)
% (objective = h'Qh- 2q'h)
nb = length(m);% number of bands
Qtvec = zeros(N,1);Qhcvec=zeros (N , 1) ;Qhrvec=zeros (N , 1) ;
q = zeros(N, 1);
n = (0:N-1)';
for i=1:nb,
    fu = om(2*i)/pi;
    fl = om(2*i-1)/pi;
    if m(i),
        Qtvec = Qtvec + ...
            (wm(i) + wp(i)/m(i)-2)* ...
            (fu*sinc(n*fu)-fl*sinc(n*fl));
        Qhcvec = Qhcvec + ...
            (wm(i) - wp(i)/m(i)-2)* ...
            (fu*sinc((n-2*tau(i))*fu)-fl*sinc((n-2*tau(i))*fl));
        Qhrvec = Qhrvec + ...
            (wm(i) - wp(i)/m(i)-2)* ...
            (fu*sinc((n+N-1-2*tau(i))*fu)-fl*sinc((n+N-1-2*tau(i))*fl));
        q = q + ...
            wm(i)*m(i)*2*(fu*sinc((n-tau(i))*fu) - fl*sinc((n-tau(i))*fl));
    else
        Qtvec = Qtvec + wm(i)*2*(fu*sinc(n*fu)-fl*sinc(n*fl));
    end
end
Q = toeplitz(Qtvec) + hankel(Qhcvec,Qhrvec);
% Compute WLS solution
h = Q\q;