function [x,l] = nnqmin(H,f,xO)
% NNQMIN: quadratic function minimization subject to non-negativity
% constraints on the variables:
% min O.5*x'*H*x + f'*x, S.t. x )= 0
% [x,l] = nnqmin(H,f,xO) returns solution x and corresponding
% Lagrange multipliers 1; H is assumed to be positive definite
% x0: optional starting guess
%
% Author: Mathias C. Lang, Vienna University of Technology, July '98

f = f(:); n = length(f);
if nargin == 3,
    x=xO(:);
    if length(x) ~= n, error('x0 has wrong length I '); end
    Ix = find(x < 0);
    if ~isempty(Ix), x(Ix)=O; end
else
    x = zeros(n,1);
end

I = find(x>0);  % indices of inactive constraints
xI = x(I);

while 1,
    if ~isempty(I), alpha = 0;
    else alpha = 1; end
    
    % solve equality constrained problem
    while alpha < 1,

        s = -(xI + H(I,I)\f(I)); % search direction
        xnew = xI + s;
        if min(xnew) < 0, % new constraint becomes active
            Is = find(s < 0);
            [alpha,p]=min(-xI(Is)./s(Is)); p=Is(p);
            I(p)=[];
            xI = xI + alpha*s; xI(p)=[];
        else
            xI = xnew;
            alpha = 1;
        end
    end
    
    % compute Lagrange multipliers
    if ~isempty(I),
        l = H(: ,I)*xI + f; l(I) = 0;
    else
        l = f;
    end
    
    [minl,q] = min(l);
    if minl < 0, % new constraint becomes inactive
        I = [I; q] ;
        xI=[xI;0];
    else         % optimum found!
        x = zeros(n,1); x(I) = xI;
        break;
    end
end
