function [delta,how] = update(J,E,a,r,M,N)
% subroutine for mpiir_12.m (IIR filter design)
% computes solution update subject to constraint on pole radii;
% applies Rouche's Theorem
% J     Jacobian of actual frequency response
% E     complex error
% a     actual denominator coefficients
% r    maximum pole radius
% M     order of numerator polynomial
% N     order of denominator polynomial
%
% Author: Mathias C. Lang, Vienna University of Technology, Oct. 98

tol = 1e-6; se = sqrt(eps); itmax = 10; cnt = 0; how = 1;%'ok';%1;
nfft = 2^10; om = 2*pi*(0:nfft/2)'/nfft;
a = a.*(r.^-(0:N)'); A = abs(fft(a,nfft)); A = A(1:nfft/2+1); A=A(:);
ra = conv(a,a(N+1:-1:1)); ra = ra(N+1:2*N+1);
nra = (1 :N)' .*ra(2:N+1); n2ra = (1:N)'.*nra;
R = r.^-(1:N); Bact=[]; cact=[];

%opts = optimset('LargeScale','off','Display','off');

% solve unconstrained problem
J = [real(J);imag(J)]; E = [real(E);imag(E)];
H=J'*J; H=H+se*eye(size(H)); f=J'*E; delta=H\f;

% delta

% compute matrices and vectors for qp-subproblems
H11=H(1:N,1:N); H12=H(1:N,N+1:M+N+1); H22=H(N+1:M+N+1,N+1:M+N+1);
Hh=H12/H22; H=H11-Hh*H12'; H=.5*(H+H');
f1=f(1:N); f2=f(N+1:M+N+1); f=Hh*f2-f1;

% iterate
while cnt<50,

	% compute error maxima on a grid
	d = delta(1:N); d = d.*(r.^-(1:N)');
	D = abs(fft(d,nfft)); D = D(1:nfft/2+1); D=D(:);
	Imax = locmax(D.^2-A.^2); ommax = om(Imax);
	if ~length(Imax), break; end
    
    % refine maxima using Newton's method
	if N>1,
		rd = conv(d,d(N:-1:1)); rd = rd(N:2*N-1);
		nrd = (1:N-1)'.*rd(2:N); n2rd = (1:N-1)'.*nrd;
	end
    for i=1:itmax,
		Mc = cos(ommax*(1:N)); Ms = sin(ommax*(1:N));
		gp = -Ms*nra; gpp = -Mc*n2ra;
		if N>1, gp = gp + Ms(:,1:N-1)*nrd; gpp = gpp + Mc(: , 1 :N-1)*n2rd; end
		Ipp = find(gpp); if ~length(Ipp), break; end
		ommax(Ipp) = ommax(Ipp)- gp(Ipp)./gpp(Ipp);
    end
    
	% find violating maxima
	Dmax = exp(-j*ommax*(1:N))*d; Amax = exp(-j*ommax*(0:N))*a;
	Iviol = find(abs(Dmax)>abs(Amax));
	omviol = ommax(Iviol); nviol = length(Iviol);
	Dviol = Dmax(Iviol); Aviol = Amax(Iviol);
    
	% check stopping criterion
	if (~nviol) | (all(abs(Dmax)-abs(Amax)<=max(abs(Amax)*tol,se))), break; end
    cnt = cnt+1;

    % formulate new constraints
	PDviol  = angle(Dviol);
	B = R(ones(nviol,1),: ) .*cos(omviol*(1:N)+PDviol(:,ones(N,1)));
	c  =  abs(Aviol);
    
	% solve   subproblem
	B= [Bact;B];   c=[cact;c];
% 	[delta,fval,how,outstruct,lamstruct]=quadprog(H,f,B,c,[],[],[],[],[],opts);
%     lam=lamstruct.ineqlin;
%	if  (how<=0),   break;   end

%size(H), size(f), size(B), size(c)
     [delta,fval,how,lam]=qp([],H,f,[],[],[],[],[],B,c);
     how=how.info;
   
 if  (how>2),   break;   end

	% find active constraints
	act   =  find(lam>0);   Bact  = B(act,:);   cact   =  c(act);
    
end

% add numerator coefficient update
if  length(delta)==N,   delta=[delta;H22\f2-Hh'*delta];   end
